package io.github.null2264.tsukumogami.core

import dev.kord.core.Kord
import dev.kord.core.event.gateway.ReadyEvent
import dev.kord.core.event.message.MessageCreateEvent
import dev.kord.core.on
import io.github.null2264.tsukumogami.core.ext.loadModule
import io.github.null2264.tsukumogami.core.koin.TsukumogamiKoinContext
import io.github.null2264.tsukumogami.core.module.BotModule
import kotlin.reflect.KFunction
import kotlinx.coroutines.runBlocking
import org.koin.core.module.Module as KoinModule
import org.koin.dsl.bind

class BotBuilder internal constructor(
    val bot: Bot
) {

    var token: String = ""

    var kordBuilder: suspend (String) -> Kord = { token ->
        Kord(token).apply {
            on<ReadyEvent> { bot.onReady() }
            on<MessageCreateEvent> { bot.onMessage(this.message, this) }
        }
    }

    internal var earlyKoinModules = mutableListOf<KoinModule>()
    internal var koinModules = mutableListOf<KoinModule>()

    fun modules(vararg modules: BotModule) {
        modules.forEach(bot::addModule)
    }

    fun prefixes(vararg prefixes: String) {
        prefixes.forEach(bot::addPrefix)
    }

    fun earlyKoinModules(vararg modules: KoinModule) {
        earlyKoinModules.addAll(modules)
    }

    fun koinModules(vararg modules: KoinModule) {
        koinModules.addAll(modules)
    }
}

suspend fun bot(clazz: KFunction<Bot> = ::Bot, declaration: BotBuilder.() -> Unit): Bot {
    if (TsukumogamiKoinContext.getOrNull() == null)
        TsukumogamiKoinContext.startKoin {
        }

    val bot = clazz.call()
    val holder = BotBuilder(bot)
    declaration(holder)

    holder.earlyKoinModules.forEach(::loadModule)

    val kord = holder.kordBuilder(holder.token)
    loadModule { single { kord } bind Kord::class }
    loadModule { single { bot } bind Bot::class }

    holder.koinModules.forEach(::loadModule)

    return holder.bot
}
