package io.github.null2264.tsukumogami.core

import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.entity.Message
import dev.kord.rest.builder.message.AllowedMentionsBuilder
import dev.kord.rest.builder.message.allowedMentions
import io.github.null2264.tsukumogami.core.commands.Command
import io.github.null2264.tsukumogami.core.koin.TsukumogamiKoinComponent
import org.koin.core.component.inject

class Context(
    val message: Message,
    /**
     * The prefix that used to invoke the command
     */
    val prefix: String?,
) : TsukumogamiKoinComponent {

    val bot: Bot by inject()

    /**
     * The user that invoked the command
     *
     * Alias to [Message.author]
     */
    val author get() = message.author

    /**
     * The text that invoked the command
     */
    var invokedWith: String? = null

    /**
     * The command that currently being invoked
     */
    var command: Command? = null

    suspend fun send(content: String) = message.channel.createMessage(content)

    suspend fun reply(content: String, mentionsAuthor: Boolean = false) = message.channel.createMessage {
        this.content = content
        messageReference = message.id
        if (!mentionsAuthor)
            allowedMentions {
                AllowedMentionsBuilder().repliedUser
            }
    }

    suspend fun typing() = message.channel.type()

    fun parseArguments(): MutableList<String>? =
        if (prefix != null && invokedWith != null) {
            message.content
                .substringAfter("$prefix$invokedWith")
                .trim()
                .split(' ')
                .toMutableList()
        } else {
            null
        }
}
